@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -74,7 +74,7 @@ class DescopeVisitor final : public VNVisitor {
             string name = scopep->name();
             string::size_type pos;
             if ((pos = name.rfind('.')) != string::npos) name.erase(0, pos + 1);
-            ret.thisPtr = VSelfPointerText{VSelfPointerText::This(), name};
+            ret.thisPtr = VSelfPointerText{VSelfPointerText::This{}, name};
         }
         return ret.thisPtr;
     }
@@ -104,9 +104,9 @@ class DescopeVisitor final : public VNVisitor {
         if (VN_IS(scopep->modp(), Class)) {
             // Direct reference to class members are from within the class itself, references from
             // outside the class must go via AstMemberSel
-            return VSelfPointerText{VSelfPointerText::This()};
+            return VSelfPointerText{VSelfPointerText::This{}};
         } else if (relativeRefOk && scopep == m_scopep) {
-            return VSelfPointerText{VSelfPointerText::This()};
+            return VSelfPointerText{VSelfPointerText::This{}};
         } else if (relativeRefOk && !m_modSingleton && scopep->aboveScopep() == m_scopep
                    && VN_IS(scopep->modp(), Module)) {
             // Reference to scope of instance directly under this module, can just "this->cell",
@@ -208,18 +208,16 @@ class DescopeVisitor final : public VNVisitor {
     // VISITORS
     void visit(AstNodeModule* nodep) override {
         VL_RESTORER(m_modp);
-        {
-            m_modp = nodep;
-            m_modFuncs.clear();
-            m_modSingleton = modIsSingleton(m_modp);
-            iterateChildren(nodep);
-            makePublicFuncWrappers();
-        }
+        m_modp = nodep;
+        m_modFuncs.clear();
+        m_modSingleton = modIsSingleton(m_modp);
+        iterateChildren(nodep);
+        makePublicFuncWrappers();
     }
     void visit(AstScope* nodep) override {
+        VL_RESTORER(m_scopep);
         m_scopep = nodep;
         iterateChildren(nodep);
-        m_scopep = nullptr;
     }
     void visit(AstVarScope* nodep) override {
         // Delete the varscope when we're finished
