@@ -287,6 +287,7 @@ class EmitCHeader final : public EmitCConstInit {
             putns(itemp, itemp->dtypep()->cType(itemp->nameProtect(), false, false));
             puts(";\n");
         }
+
         // Three helper functions for struct constrained randomization:
         // - memberNames: Get member names
         // - getMembers: Access member references
@@ -294,13 +295,15 @@ class EmitCHeader final : public EmitCConstInit {
         // - memberWidth: Retrieve member width
         // - memberDimension: Retrieve member dimension
         if (sdtypep->isConstrainedRand()) {
+            bool needComma = false;
             putns(sdtypep, "\nstd::vector<std::string> memberNames(void) const {\n");
             puts("return {");
             for (const AstMemberDType* itemp = sdtypep->membersp(); itemp;
                  itemp = VN_AS(itemp->nextp(), MemberDType)) {
-                if (itemp->isConstrainedRand()) putns(itemp, "\"" + itemp->shortName() + "\"");
-                if (itemp->nextp() && VN_AS(itemp->nextp(), MemberDType)->isConstrainedRand())
-                    puts(",\n");
+                if (!itemp->isConstrainedRand()) continue;
+                if (needComma) puts(",\n");
+                putns(itemp, "\"" + itemp->shortName() + "\"");
+                needComma = true;
             }
             puts("};\n}\n");
 
@@ -310,25 +313,28 @@ class EmitCHeader final : public EmitCConstInit {
             putns(sdtypep, "\nstd::vector<int> memberDimension(void) const {\n");
             emitMemberVector<AttributeType::Dimension>(sdtypep);
 
+            needComma = false;
             putns(sdtypep, "\nauto memberIndices(void) const {\n");
             puts("return std::index_sequence_for<");
             for (const AstMemberDType* itemp = sdtypep->membersp(); itemp;
                  itemp = VN_AS(itemp->nextp(), MemberDType)) {
-                if (itemp->isConstrainedRand())
-                    putns(itemp, itemp->dtypep()->cType("", false, false));
-                if (itemp->nextp() && VN_AS(itemp->nextp(), MemberDType)->isConstrainedRand())
-                    puts(",\n");
+                if (!itemp->isConstrainedRand()) continue;
+                if (needComma) puts(",\n");
+                putns(itemp, itemp->dtypep()->cType("", false, false));
+                needComma = true;
             }
             puts(">{};\n}\n");
 
+            needComma = false;
             putns(sdtypep, "\ntemplate <typename T>");
             putns(sdtypep, "\nauto getMembers(T& obj) {\n");
             puts("return std::tie(");
             for (const AstMemberDType* itemp = sdtypep->membersp(); itemp;
                  itemp = VN_AS(itemp->nextp(), MemberDType)) {
-                if (itemp->isConstrainedRand()) putns(itemp, "obj." + itemp->nameProtect());
-                if (itemp->nextp() && VN_AS(itemp->nextp(), MemberDType)->isConstrainedRand())
-                    puts(", ");
+                if (!itemp->isConstrainedRand()) continue;
+                if (needComma) puts(",\n");
+                putns(itemp, "obj." + itemp->nameProtect());
+                needComma = true;
             }
             puts(");\n}\n");
         }
@@ -595,6 +601,7 @@ class EmitCHeader final : public EmitCConstInit {
         // Emit out of class function declarations
         puts("\n");
         emitFuncDecls(modp, /* inClassBody: */ false);
+        emitTextSection(modp, VNType::atScHdrPost);
     }
 
     explicit EmitCHeader(const AstNodeModule* modp) {
