@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -24,10 +24,12 @@
 // FORKs that spawn tasks which might outlive their parents require those
 // tasks to carry their own frames and as such they require their own
 // variable scopes.
+//
 // There are two mechanisms that work together to achieve that. ForkVisitor
 // moves bodies of forked processes into new tasks, which results in them getting their
 // own scopes. The original statements get replaced with a call to the task which
 // passes the required variables by value.
+//
 // The second mechanism, DynScopeVisitor, is designed to handle variables which can't be
 // captured by value and instead require a reference. Those variables get moved into an
 // "anonymous" object, ie. a class with appropriate fields gets generated and an object
@@ -330,7 +332,7 @@ class DynScopeVisitor final : public VNVisitor {
             membersel->varp(refp->varp());
         }
         handle.relink(membersel);
-        VL_DO_DANGLING(refp->deleteTree(), refp);
+        VL_DO_DANGLING(pushDeletep(refp), refp);
     }
 
     static bool hasAsyncFork(AstNode* nodep) {
@@ -433,7 +435,7 @@ class DynScopeVisitor final : public VNVisitor {
                 nodep->v3warn(
                     E_UNSUPPORTED,
                     "Unsupported: Writing to a captured "
-                        << (nodep->varp()->isInoutish() ? "inout" : "output") << " variable in a "
+                        << (nodep->varp()->isInout() ? "inout" : "output") << " variable in a "
                         << (VN_IS(nodep->backp(), AssignDly) ? "non-blocking assignment" : "fork")
                         << " after a timing control");
             }
@@ -591,7 +593,7 @@ class ForkVisitor final : public VNVisitor {
             taskp
                 = makeTask(beginp->fileline(), beginp->stmtsp()->unlinkFrBackWithNext(), taskName);
             beginp->unlinkFrBack(&handle);
-            VL_DO_DANGLING(beginp->deleteTree(), beginp);
+            VL_DO_DANGLING(pushDeletep(beginp), beginp);
         } else if (AstNodeStmt* const stmtp = VN_CAST(nodep, NodeStmt)) {
             const string taskName = generateTaskName(stmtp, "fork_stmt");
             taskp = makeTask(stmtp->fileline(), stmtp->unlinkFrBack(&handle), taskName);
@@ -603,9 +605,7 @@ class ForkVisitor final : public VNVisitor {
         m_modp->addStmtsp(taskp);
         UINFO(9, "new " << taskp << endl);
 
-        AstTaskRef* const taskrefp
-            = new AstTaskRef{nodep->fileline(), taskp->name(), m_capturedVarRefsp};
-        taskrefp->taskp(taskp);
+        AstTaskRef* const taskrefp = new AstTaskRef{nodep->fileline(), taskp, m_capturedVarRefsp};
         AstStmtExpr* const taskcallp = taskrefp->makeStmt();
         // Replaced nodes will be revisited, so we don't need to "lift" the arguments
         // as captures in case of nested forks.
@@ -619,7 +619,7 @@ class ForkVisitor final : public VNVisitor {
 
         VL_RESTORER(m_forkLocalsp);
         VL_RESTORER(m_newProcess);
-        VL_RESTORER(m_forkDepth)
+        VL_RESTORER(m_forkDepth);
         if (!nodep->joinType().join()) {
             ++m_forkDepth;
             m_newProcess = true;
@@ -644,8 +644,7 @@ class ForkVisitor final : public VNVisitor {
         if (m_forkDepth && !nodep->varp()->isFuncLocal() && nodep->varp()->isClassMember()) return;
 
         if (m_forkDepth && (m_forkLocalsp.count(nodep->varp()) == 0)
-            && nodep->varp()->varType() != VVarType::PORT  // Basically static, so it's safe
-            && !nodep->varp()->lifetime().isStatic()) {
+            && !nodep->varp()->lifetime().isStatic()) {  // Basically static, so it's safe
             if (nodep->access().isWriteOrRW()
                 && (!nodep->isClassHandleValue() || nodep->user2())) {
                 nodep->v3warn(
@@ -682,7 +681,7 @@ class ForkVisitor final : public VNVisitor {
         iterateChildren(nodep);
     }
     void visit(AstNode* nodep) override {
-        VL_RESTORER(m_newProcess)
+        VL_RESTORER(m_newProcess);
         VL_RESTORER(m_forkDepth);
         if (nodep->user1()) --m_forkDepth;
         m_newProcess = false;
