@@ -195,3 +195,19 @@ const std::string& V3Global::ptrToId(const void* p) {
     }
     return pair.first->second;
 }
+
+std::vector<std::string> V3Global::verilatedCppFiles() {
+    std::vector<std::string> result;
+    result.emplace_back("verilated.cpp");
+    if (v3Global.dpi()) result.emplace_back("verilated_dpi.cpp");
+    if (v3Global.opt.vpi()) result.emplace_back("verilated_vpi.cpp");
+    if (v3Global.opt.savable()) result.emplace_back("verilated_save.cpp");
+    if (v3Global.opt.coverage()) result.emplace_back("verilated_cov.cpp");
+    if (v3Global.opt.trace()) result.emplace_back(v3Global.opt.traceSourceBase() + "_c.cpp");
+    if (v3Global.usesProbDist()) result.emplace_back("verilated_probdist.cpp");
+    if (v3Global.usesTiming()) result.emplace_back("verilated_timing.cpp");
+    if (v3Global.useRandomizeMethods()) result.emplace_back("verilated_random.cpp");
+    result.emplace_back("verilated_threads.cpp");
+    if (v3Global.opt.usesProfiler()) result.emplace_back("verilated_profiler.cpp");
+    return result;
+}
