@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -109,6 +109,7 @@ class InlineMarkVisitor final : public VNVisitor {
     // VISITORS
     void visit(AstNodeModule* nodep) override {
         UASSERT_OBJ(!m_modp, nodep, "Unsupported: Nested modules");
+        VL_RESTORER(m_modp);
         m_modp = nodep;
         m_allMods.push_back(nodep);
         m_modp->user2(CIL_MAYBE);
@@ -123,7 +124,6 @@ class InlineMarkVisitor final : public VNVisitor {
         }
 
         iterateChildren(nodep);
-        m_modp = nullptr;
     }
     void visit(AstClass* nodep) override {
         // TODO allow inlining of modules that have classes
@@ -252,6 +252,7 @@ class InlineRelinkVisitor final : public VNVisitor {
     std::unordered_set<std::string> m_renamedInterfaces;  // Name of renamed interface variables
     AstNodeModule* const m_modp;  // Current module
     const AstCell* const m_cellp;  // Cell being cloned
+    bool m_initialStatic = false;  // Inside InitialStatic
 
     // VISITORS
     void visit(AstCellInline* nodep) override {
@@ -290,6 +291,7 @@ class InlineRelinkVisitor final : public VNVisitor {
             if (exprconstp) {
                 m_modp->addStmtsp(new AstAssignW{flp, new AstVarRef{flp, nodep, VAccess::WRITE},
                                                  exprconstp->cloneTree(false)});
+                nodep->user4(true);  // Making assignment to it
             } else if (nodep->user3()) {
                 // Public variable at the lower module end - we need to make sure we propagate
                 // the logic changes up and down; if we aliased, we might
@@ -305,6 +307,7 @@ class InlineRelinkVisitor final : public VNVisitor {
                 UINFO(9, "assign to public and unpacked: " << nodep << endl);
                 exprvarrefp = exprvarrefp->cloneTree(false);
                 exprvarrefp->access(VAccess::READ);
+                nodep->user4(true);  // Making assignment to it
                 m_modp->addStmtsp(
                     new AstAssignW{flp, new AstVarRef{flp, nodep, VAccess::WRITE}, exprvarrefp});
             } else if (nodep->isIfaceRef()) {
@@ -322,6 +325,7 @@ class InlineRelinkVisitor final : public VNVisitor {
                 exprvarrefp->access(VAccess::READ);
                 AstVarRef* const nodeVarRefp = new AstVarRef{flp, nodep, VAccess::WRITE};
                 if (nodep->isForced() && nodep->direction() == VDirection::INPUT) {
+                    nodep->user4(true);  // Making assignment to it
                     m_modp->addStmtsp(new AstAssignW{flp, nodeVarRefp, exprvarrefp});
                 } else if (nodep->isForced() && nodep->direction() == VDirection::OUTPUT) {
                     exprvarrefp->access(VAccess::WRITE);
@@ -374,6 +378,22 @@ class InlineRelinkVisitor final : public VNVisitor {
         nodep->name(m_cellp->name() + "__DOT__" + nodep->name());
         iterateChildren(nodep);
     }
+    void visit(AstInitialStatic* nodep) override {
+        VL_RESTORER(m_initialStatic);
+        m_initialStatic = true;
+        iterateChildren(nodep);
+    }
+    void visit(AstNodeAssign* nodep) override {
+        if (AstVarRef* const varrefp = VN_CAST(nodep->lhsp(), VarRef)) {
+            if (m_initialStatic && varrefp->varp()->user2() && varrefp->varp()->user4()) {
+                // Initial assignment to i/o we are overriding, can remove
+                UINFO(9, "Remove InitialStatic " << nodep << endl);
+                VL_DO_DANGLING(pushDeletep(nodep->unlinkFrBack()), nodep);
+                return;
+            }
+        }
+        iterateChildren(nodep);
+    }
     void visit(AstVarRef* nodep) override {
         if (nodep->varp()->user2p()  // It's being converted to an alias.
             && !nodep->varp()->user3()
@@ -470,6 +490,7 @@ class InlineVisitor final : public VNVisitor {
     //   AstVar::user2p()       // AstVarRef*/AstConst*  Points to signal this
     //                          // is a direct connect to
     //   AstVar::user3()        // bool    Don't alias the user2, keep it as signal
+    //   AstVar::user4()        // bool    Was input, remove InitialStatic Assign
     //   AstCell::user4         // AstCell* of the created clone
     const VNUser4InUse m_inuser4;
 
@@ -584,13 +605,13 @@ class InlineVisitor final : public VNVisitor {
     }
     void visit(AstNodeModule* nodep) override {
         UASSERT_OBJ(!m_modp, nodep, "Unsupported: Nested modules");
+        VL_RESTORER(m_modp);
         m_modp = nodep;
         // Iterate the stored cells directly to reduce traversal
         for (AstCell* const cellp : m_moduleState(nodep).m_childCells) {
             if (m_moduleState(cellp->modp()).m_inlined) inlineCell(cellp);
         }
         m_moduleState(nodep).m_childCells.clear();
-        m_modp = nullptr;
     }
     void visit(AstIfaceRefDType* nodep) override {
         if (nodep->user1()) {
@@ -603,10 +624,10 @@ class InlineVisitor final : public VNVisitor {
 
     //--------------------
     void visit(AstCell* nodep) override {  // LCOV_EXCL_START
-        nodep->v3fatal("Traversal should have been short circuited");
+        nodep->v3fatalSrc("Traversal should have been short circuited");
     }
     void visit(AstNodeStmt* nodep) override {
-        nodep->v3fatal("Traversal should have been short circuited");
+        nodep->v3fatalSrc("Traversal should have been short circuited");
     }  // LCOV_EXCL_STOP
     void visit(AstNodeFile*) override {}  // Accelerate
     void visit(AstNodeDType*) override {}  // Accelerate
