@@ -474,7 +474,7 @@ void V3Options::decorations(FileLine* fl, const string& arg) {  // --decorations
         m_decoration = true;
         m_decorationNodes = false;
     } else {
-        fl->v3fatal("Unknown setting for --decorations: '"
+        fl->v3error("Unknown setting for --decorations: '"
                     << arg << "'\n"
                     << fl->warnMore() << "... Suggest 'none', 'medium', or 'node'");
     }
@@ -506,7 +506,7 @@ string V3Options::fileExists(const string& filename) {
         m_impp->m_dirMap.emplace(dir, std::set<string>());
         diriter = m_impp->m_dirMap.find(dir);
 
-        std::set<string>* setp = &(diriter->second);
+        std::set<string>* const setp = &(diriter->second);
 
 #ifdef _MSC_VER
         try {
@@ -524,13 +524,11 @@ string V3Options::fileExists(const string& filename) {
 #endif
     }
     // Find it
-    const std::set<string>* filesetp = &(diriter->second);
+    const std::set<string>* const filesetp = &(diriter->second);
     const auto fileiter = filesetp->find(basename);
-    if (fileiter == filesetp->end()) {
-        return "";  // Not found
-    }
+    if (fileiter == filesetp->end()) return "";  // Not found
     // Check if it is a directory, ignore if so
-    string filenameOut = V3Os::filenameJoin(dir, basename);
+    const string filenameOut = V3Os::filenameJoin(dir, basename);
     if (!fileStatNormal(filenameOut)) return "";  // Directory
     return filenameOut;
 }
@@ -538,7 +536,7 @@ string V3Options::fileExists(const string& filename) {
 string V3Options::filePathCheckOneDir(const string& modname, const string& dirname) {
     for (const string& i : m_impp->m_libExtVs) {
         const string fn = V3Os::filenameJoin(dirname, modname + i);
-        string exists = fileExists(fn);
+        const string exists = fileExists(fn);
         if (exists != "") return exists;
     }
     return "";
@@ -578,15 +576,15 @@ string V3Options::filePath(FileLine* fl, const string& modname, const string& la
     const string filename = V3Os::filenameCleanup(VName::dehash(modname));
     if (!V3Os::filenameIsRel(filename)) {
         // filename is an absolute path, so can find getStdPackagePath()/getStdWaiverPath()
-        string exists = filePathCheckOneDir(filename, "");
+        const string exists = filePathCheckOneDir(filename, "");
         if (exists != "") return exists;
     }
     for (const string& dir : m_impp->m_incDirUsers) {
-        string exists = filePathCheckOneDir(filename, dir);
+        const string exists = filePathCheckOneDir(filename, dir);
         if (exists != "") return exists;
     }
     for (const string& dir : m_impp->m_incDirFallbacks) {
-        string exists = filePathCheckOneDir(filename, dir);
+        const string exists = filePathCheckOneDir(filename, dir);
         if (exists != "") return exists;
     }
 
@@ -851,7 +849,7 @@ void V3Options::notify() VL_MT_DISABLED {
                 "--xml-only, --json-only or --E option");
     }
 
-    if (m_build && (m_gmake || m_cmake)) {
+    if (m_build && (m_gmake || m_cmake || m_makeJson)) {
         cmdfl->v3error("--make cannot be used together with --build. Suggest see manual");
     }
 
@@ -883,7 +881,7 @@ void V3Options::notify() VL_MT_DISABLED {
     }
 
     // Make sure at least one make system is enabled
-    if (!m_gmake && !m_cmake) m_gmake = true;
+    if (!m_gmake && !m_cmake && !m_makeJson) m_gmake = true;
 
     if (m_hierarchical && (m_hierChild || !m_hierBlocks.empty())) {
         cmdfl->v3error(
@@ -932,7 +930,7 @@ void V3Options::notify() VL_MT_DISABLED {
     }
 
     if (trace()) {
-        // With --trace, --trace-threads is ignored
+        // With --trace-vcd, --trace-threads is ignored
         if (traceFormat().vcd()) m_traceThreads = 1;
     }
 
@@ -962,6 +960,7 @@ void V3Options::notify() VL_MT_DISABLED {
 
     // Sanity check of expected configuration
     UASSERT(threads() >= 1, "'threads()' must return a value >= 1");
+    if (m_outputGroups == -1) m_outputGroups = (m_buildJobs != -1) ? m_buildJobs : 0;
     if (m_buildJobs == -1) m_buildJobs = 1;
     if (m_verilateJobs == -1) m_verilateJobs = 1;
 
@@ -1050,7 +1049,7 @@ string V3Options::argString(int argc, char** argv) {
 
 void V3Options::parseOpts(FileLine* fl, int argc, char** argv) VL_MT_DISABLED {
     // Save command line options
-    for (int i = 0; i < argc; ++i) { addLineArg(argv[i]); }
+    for (int i = 0; i < argc; ++i) addLineArg(argv[i]);
 
     // Parse all options
     // Initial entry point from Verilator.cpp
@@ -1197,7 +1196,7 @@ void V3Options::parseOptsList(FileLine* fl, const string& optdir, int argc,
     DECL_OPTION("-build-jobs", CbVal, [this, fl](const char* valp) {
         int val = std::atoi(valp);
         if (val < 0) {
-            fl->v3fatal("--build-jobs requires a non-negative integer, but '" << valp
+            fl->v3error("--build-jobs requires a non-negative integer, but '" << valp
                                                                               << "' was passed");
             val = 1;
         } else if (val == 0) {
@@ -1230,7 +1229,7 @@ void V3Options::parseOptsList(FileLine* fl, const string& optdir, int argc,
             m_compLimitMembers = 0;  // probably ok, and AFAIK doesn't support anon structs
             m_compLimitParens = 80;  // 128, but allow some room
         } else {
-            fl->v3fatal("Unknown setting for --compiler: '"
+            fl->v3error("Unknown setting for --compiler: '"
                         << valp << "'\n"
                         << fl->warnMore() << "... Suggest 'clang', 'gcc', or 'msvc'");
         }
@@ -1355,6 +1354,7 @@ void V3Options::parseOptsList(FileLine* fl, const string& optdir, int argc,
     DECL_OPTION("-fsubst-const", FOnOff, &m_fSubstConst);
     DECL_OPTION("-ftable", FOnOff, &m_fTable);
     DECL_OPTION("-ftaskify-all-forked", FOnOff, &m_fTaskifyAll).undocumented();  // Debug
+    DECL_OPTION("-fvar-split", FOnOff, &m_fVarSplit);
 
     DECL_OPTION("-G", CbPartialMatch, [this](const char* optp) { addParameter(optp, false); });
     DECL_OPTION("-gate-stmts", Set, &m_gateStmts);
@@ -1417,7 +1417,7 @@ void V3Options::parseOptsList(FileLine* fl, const string& optdir, int argc,
             for (int i = V3LangCode::L_ERROR + 1; i < V3LangCode::_ENUM_END; ++i) {
                 spell.pushCandidate(V3LangCode{i}.ascii());
             }
-            fl->v3fatal("Unknown language specified: " << valp << spell.bestCandidateMsg(valp));
+            fl->v3error("Unknown language specified: " << valp << spell.bestCandidateMsg(valp));
         }
     };
     DECL_OPTION("-default-language", CbVal, setLang);
@@ -1443,8 +1443,10 @@ void V3Options::parseOptsList(FileLine* fl, const string& optdir, int argc,
             m_cmake = true;
         } else if (!std::strcmp(valp, "gmake")) {
             m_gmake = true;
+        } else if (!std::strcmp(valp, "json")) {
+            m_makeJson = true;
         } else {
-            fl->v3fatal("Unknown --make system specified: '" << valp << "'");
+            fl->v3error("Unknown --make system specified: '" << valp << "'");
         }
     });
     DECL_OPTION("-max-num-width", Set, &m_maxNumWidth);
@@ -1464,7 +1466,7 @@ void V3Options::parseOptsList(FileLine* fl, const string& optdir, int argc,
     });
     DECL_OPTION("-output-groups", CbVal, [this, fl](const char* valp) {
         m_outputGroups = std::atoi(valp);
-        if (m_outputGroups < 0) { fl->v3error("--output-groups must be >= 0: " << valp); }
+        if (m_outputGroups < -1) fl->v3error("--output-groups must be >= -1: " << valp);
     });
     DECL_OPTION("-output-split", Set, &m_outputSplit);
     DECL_OPTION("-output-split-cfuncs", CbVal, [this, fl](const char* valp) {
@@ -1485,7 +1487,7 @@ void V3Options::parseOptsList(FileLine* fl, const string& optdir, int argc,
     DECL_OPTION("-no-pins64", CbCall, [this]() { m_pinsBv = 33; });
     DECL_OPTION("-pins-bv", CbVal, [this, fl](const char* valp) {
         m_pinsBv = std::atoi(valp);
-        if (m_pinsBv > 65) fl->v3fatal("--pins-bv maximum is 65: " << valp);
+        if (m_pinsBv > 65) fl->v3error("--pins-bv maximum is 65: " << valp);
     });
     DECL_OPTION("-pins-inout-enables", OnOff, &m_pinsInoutEnables);
     DECL_OPTION("-pins-sc-uint", CbOnOff, [this](bool flag) {
@@ -1606,7 +1608,7 @@ void V3Options::parseOptsList(FileLine* fl, const string& optdir, int argc,
             m_threadsDpiPure = true;
             m_threadsDpiUnpure = false;
         } else {
-            fl->v3fatal("Unknown setting for --threads-dpi: '"
+            fl->v3error("Unknown setting for --threads-dpi: '"
                         << valp << "'\n"
                         << fl->warnMore() << "... Suggest 'all', 'none', or 'pure'");
         }
@@ -1639,6 +1641,10 @@ void V3Options::parseOptsList(FileLine* fl, const string& optdir, int argc,
     DECL_OPTION("-top", Set, &m_topModule);
     DECL_OPTION("-top-module", Set, &m_topModule);
     DECL_OPTION("-trace", OnOff, &m_trace);
+    DECL_OPTION("-trace-saif", CbCall, [this]() {
+        m_trace = true;
+        m_traceFormat = TraceFormat::SAIF;
+    });
     DECL_OPTION("-trace-coverage", OnOff, &m_traceCoverage);
     DECL_OPTION("-trace-depth", Set, &m_traceDepth);
     DECL_OPTION("-trace-fst", CbCall, [this]() {
@@ -1665,6 +1671,10 @@ void V3Options::parseOptsList(FileLine* fl, const string& optdir, int argc,
     });
     DECL_OPTION("-no-trace-top", Set, &m_noTraceTop);
     DECL_OPTION("-trace-underscore", OnOff, &m_traceUnderscore);
+    DECL_OPTION("-trace-vcd", CbCall, [this]() {
+        m_trace = true;
+        m_traceFormat = TraceFormat::VCD;
+    });
 
     DECL_OPTION("-U", CbPartialMatch, &V3PreShell::undef);
     DECL_OPTION("-underline-zero", OnOff, &m_underlineZero);  // Deprecated
@@ -1790,7 +1800,7 @@ void V3Options::parseOptsList(FileLine* fl, const string& optdir, int argc,
         } else if (!std::strcmp(valp, "unique")) {
             m_xAssign = "unique";
         } else {
-            fl->v3fatal("Unknown setting for --x-assign: '"
+            fl->v3error("Unknown setting for --x-assign: '"
                         << valp << "'\n"
                         << fl->warnMore() << "... Suggest '0', '1', 'fast', or 'unique'");
         }
@@ -1803,14 +1813,20 @@ void V3Options::parseOptsList(FileLine* fl, const string& optdir, int argc,
         } else if (!std::strcmp(valp, "unique")) {
             m_xInitial = "unique";
         } else {
-            fl->v3fatal("Unknown setting for --x-initial: '"
+            fl->v3error("Unknown setting for --x-initial: '"
                         << valp << "'\n"
                         << fl->warnMore() << "... Suggest '0', 'fast', or 'unique'");
         }
     });
     DECL_OPTION("-x-initial-edge", OnOff, &m_xInitialEdge);
-    DECL_OPTION("-xml-only", OnOff, &m_xmlOnly);
-    DECL_OPTION("-xml-output", CbVal, [this](const char* valp) {
+    DECL_OPTION("-xml-only", CbOnOff, [this, fl](bool flag) {
+        if (!m_xmlOnly && flag)
+            fl->v3warn(DEPRECATED, "Option --xml-only is deprecated, move to --json-only");
+        m_xmlOnly = flag;
+    });
+    DECL_OPTION("-xml-output", CbVal, [this, fl](const char* valp) {
+        if (!m_xmlOnly)
+            fl->v3warn(DEPRECATED, "Option --xml-only is deprecated, move to --json-only");
         m_xmlOutput = valp;
         m_xmlOnly = true;
     });
@@ -1834,6 +1850,7 @@ void V3Options::parseOptsList(FileLine* fl, const string& optdir, int argc,
             }
             if (m_buildJobs == -1) m_buildJobs = val;
             if (m_verilateJobs == -1) m_verilateJobs = val;
+            if (m_outputGroups == -1) m_outputGroups = val;
         } else if (argv[i][0] == '-' || argv[i][0] == '+') {
             const char* argvNoDashp = (argv[i][1] == '-') ? (argv[i] + 2) : (argv[i] + 1);
             if (const int consumed = parser.parse(i, argc, argv)) {
@@ -2162,6 +2179,7 @@ void V3Options::optimize(int level) {
     m_fSubst = flag;
     m_fSubstConst = flag;
     m_fTable = flag;
+    m_fVarSplit = flag;
     // And set specific optimization levels
     if (level >= 3) {
         m_inlineMult = -1;  // Maximum inlining
