@@ -57,6 +57,7 @@ VL_DEFINE_DEBUG_FUNCTIONS;
 // clang-format off
 #if defined(_WIN32) || defined(__MINGW32__)
 # include <windows.h>   // LONG for bcrypt.h on MINGW
+# include <algorithm>  // replace
 # include <bcrypt.h>  // BCryptGenRandom
 # include <chrono>
 # include <direct.h>  // mkdir
@@ -310,6 +311,16 @@ bool V3Os::filenameIsRel(const string& filename) VL_PURE {
 #endif
 }
 
+string V3Os::filenameSlashPath(const string& path) VL_PURE {
+#if defined(_WIN32) || defined(__MINGW32__)
+    string slashedPath = path;
+    std::replace(slashedPath.begin(), slashedPath.end(), '\\', '/');
+    return slashedPath;
+#else
+    return path;
+#endif
+}
+
 //######################################################################
 // File utilities
 
