@@ -211,6 +211,7 @@ private:
     string defineSubst(VDefineRef* refp);
 
     bool defExists(const string& name);
+    bool defCmdline(const string& name);
     string defValue(const string& name);
     string defParams(const string& name);
     FileLine* defFileline(const string& name);
@@ -242,7 +243,7 @@ private:
     void statePop() {
         m_states.pop();
         if (VL_UNCOVERABLE(m_states.empty())) {
-            error("InternalError: Pop of parser state with nothing on stack");  // LCOV_EXCL_LINE
+            fileline()->v3fatalSrc("Pop of parser state with nothing on stack");
             m_states.push(ps_TOP);  // LCOV_EXCL_LINE
         }
     }
@@ -316,6 +317,11 @@ bool V3PreProcImp::defExists(const string& name) {
     const auto iter = m_defines.find(name);
     return (iter != m_defines.end());
 }
+bool V3PreProcImp::defCmdline(const string& name) {
+    const auto iter = m_defines.find(name);
+    if (iter == m_defines.end()) { return false; }
+    return iter->second.cmdline();
+}
 string V3PreProcImp::defValue(const string& name) {
     const auto iter = m_defines.find(name);
     if (iter == m_defines.end()) {
@@ -345,21 +351,37 @@ void V3PreProcImp::define(FileLine* fl, const string& name, const string& value,
                                                                   << "' (IEEE 1800-2023 22.5.1)");
     } else {
         if (defExists(name)) {
-            if (!(defValue(name) == value
-                  && defParams(name) == params)) {  // Duplicate defs are OK
-                fl->v3warn(REDEFMACRO, "Redefining existing define: '"
-                                           << name << "', with different value: '" << value
-                                           << (params == "" ? "" : " ") << params << "'\n"
-                                           << fl->warnContextPrimary() << '\n'
-                                           << defFileline(name)->warnOther()
-                                           << "... Location of previous definition, with value: '"
-                                           << defValue(name)
-                                           << (defParams(name).empty() ? "" : " ")
-                                           << defParams(name) << "'\n"
-                                           << defFileline(name)->warnContextSecondary());
+            if (defCmdline(name) && !cmdline) {
+                fl->v3warn(DEFOVERRIDE,
+                           "Overriding define: '"
+                               << name << "' with value: '" << value
+                               << "' to existing command line define value: '" << defValue(name)
+                               << (params == "" ? "" : " ") << params << "'\n"
+                               << fl->warnContextPrimary() << '\n'
+                               << defFileline(name)->warnOther()
+                               << "... Location of previous definition, with value: '"
+                               << defValue(name) << (defParams(name).empty() ? "" : " ")
+                               << defParams(name) << "'\n"
+                               << defFileline(name)->warnContextSecondary());
+                return;
+            } else {
+                if (!(defValue(name) == value
+                      && defParams(name) == params)) {  // Duplicate defs are OK
+                    fl->v3warn(REDEFMACRO,
+                               "Redefining existing define: '"
+                                   << name << "', with different value: '" << value
+                                   << (params == "" ? "" : " ") << params << "'\n"
+                                   << fl->warnContextPrimary() << '\n'
+                                   << defFileline(name)->warnOther()
+                                   << "... Location of previous definition, with value: '"
+                                   << defValue(name) << (defParams(name).empty() ? "" : " ")
+                                   << defParams(name) << "'\n"
+                                   << defFileline(name)->warnContextSecondary());
+                }
+                undef(name);
             }
-            undef(name);
         }
+
         m_defines.emplace(name, VDefine{fl, value, params, cmdline});
     }
 }
@@ -640,8 +662,8 @@ string V3PreProcImp::defineSubst(VDefineRef* refp) {
                             const string arg = trimWhitespace(refp->args()[numArgs], true);
                             if (arg != "") valueDef = arg;
                         } else if (!haveDefault) {
-                            error("Define missing argument '" + argName + "' for: " + refp->name()
-                                  + "\n");
+                            fileline()->v3error("Define missing argument '" + argName
+                                                + "' for: " + refp->name());
                             return " `" + refp->name() + " ";
                         }
                         numArgs++;
@@ -679,7 +701,7 @@ string V3PreProcImp::defineSubst(VDefineRef* refp) {
             // `define X() is ok to call with nothing
             && !(refp->args().size() == 1 && numArgs == 0
                  && trimWhitespace(refp->args()[0], false) == "")) {
-            error("Define passed too many arguments: " + refp->name() + "\n");
+            fileline()->v3error("Define passed too many arguments: " + refp->name());
             return " `" + refp->name() + " ";
         }
     }
@@ -800,7 +822,7 @@ void V3PreProcImp::openFile(FileLine*, VInFilter* filterp, const string& filenam
     StrList wholefile;
     const bool ok = filterp->readWholefile(filename, wholefile /*ref*/);
     if (!ok) {
-        error("File not found: " + filename + "\n");
+        fileline()->v3error("File not found: " + filename);
         return;
     }
 
@@ -808,7 +830,7 @@ void V3PreProcImp::openFile(FileLine*, VInFilter* filterp, const string& filenam
         // We allow the same include file twice, because occasionally it pops
         // up, with guards preventing a real recursion.
         if (m_lexp->m_streampStack.size() > V3PreProc::INCLUDE_DEPTH_MAX) {
-            error("Recursive inclusion of file: " + filename);
+            fileline()->v3error("Recursive inclusion of file: " + filename);
             // Include might be a tree of includes that is O(n^2) or worse.
             // Once hit this error then, ignore all further includes so can unwind.
             m_incError = true;
@@ -964,8 +986,9 @@ int V3PreProcImp::getRawToken() {
             m_lastLineno = m_lexp->m_tokFilelinep->lineno();
             m_tokensOnLine = 0;
         } else if (++m_tokensOnLine > v3Global.opt.preprocTokenLimit()) {
-            error("Too many preprocessor tokens on a line (>"
-                  + cvtToStr(v3Global.opt.preprocTokenLimit()) + "); perhaps recursive `define");
+            fileline()->v3error("Too many preprocessor tokens on a line (>"
+                                + cvtToStr(v3Global.opt.preprocTokenLimit())
+                                + "); perhaps recursive `define");
             tok = VP_EOF_ERROR;
         }
 
@@ -1103,7 +1126,7 @@ int V3PreProcImp::getStateToken() {
                     goto next_tok;
                 } else if (state() == ps_DEFNAME_ELSIF) {
                     if (m_ifdefStack.empty()) {
-                        error("`elsif with no matching `if\n");
+                        fileline()->v3error("`elsif with no matching `if");
                     } else {
                         // Handle `else portion
                         const VPreIfEntry lastIf = m_ifdefStack.top();
@@ -1159,7 +1182,7 @@ int V3PreProcImp::getStateToken() {
                 // IE, `ifdef `MACRO(x): Substitute and come back here when state pops.
                 break;
             } else {
-                error("Expecting define name. Found: "s + tokenName(tok) + "\n");
+                fileline()->v3error("Expecting define name. Found: "s + tokenName(tok));
                 goto next_tok;
             }
         }
@@ -1208,7 +1231,7 @@ int V3PreProcImp::getStateToken() {
                         goto next_tok;
                     } else if (state() == ps_EXPR_ELSIF) {
                         if (m_ifdefStack.empty()) {
-                            error("`elsif with no matching `if\n");
+                            fileline()->v3error("`elsif with no matching `if");
                         } else {
                             // Handle `else portion
                             const VPreIfEntry lastIf = m_ifdefStack.top();
@@ -1243,8 +1266,8 @@ int V3PreProcImp::getStateToken() {
                 if (VString::removeWhitespace(string{yyourtext(), yyourleng()}).empty()) {
                     return tok;
                 } else {
-                    error("Syntax error in `ifdef () expression; unexpected: '"s + tokenName(tok)
-                          + "'\n");
+                    fileline()->v3error("Syntax error in `ifdef () expression; unexpected: '"s
+                                        + tokenName(tok) + "'");
                 }
                 goto next_tok;
             }
@@ -1266,7 +1289,8 @@ int V3PreProcImp::getStateToken() {
                     goto next_tok;
                 }
             } else {
-                error("Expecting define formal arguments. Found: "s + tokenName(tok) + "\n");
+                fileline()->v3error("Expecting define formal arguments. Found: "s + tokenName(tok)
+                                    + "\n");
                 goto next_tok;
             }
         }
@@ -1318,8 +1342,8 @@ int V3PreProcImp::getStateToken() {
             } else {
                 UASSERT(!m_defRefs.empty(), "Shouldn't be in DEFPAREN w/o active defref");
                 const VDefineRef* const refp = &(m_defRefs.top());
-                error("Expecting ( to begin argument list for define reference `"s + refp->name()
-                      + "\n");
+                fileline()->v3error("Expecting ( to begin argument list for define reference `"s
+                                    + refp->name());
                 statePop();
                 goto next_tok;
             }
@@ -1392,9 +1416,9 @@ int V3PreProcImp::getStateToken() {
                 statePush(ps_STRIFY);
                 goto next_tok;
             } else {
-                error(std::string{
-                          "Expecting ) or , to end argument list for define reference. Found: "}
-                      + tokenName(tok));
+                fileline()->v3error(
+                    "Expecting ) or , to end argument list for define reference. Found: "s
+                    + tokenName(tok));
                 statePop();
                 goto next_tok;
             }
@@ -1419,7 +1443,7 @@ int V3PreProcImp::getStateToken() {
                 break;
             } else {
                 statePop();
-                error("Expecting include filename. Found: "s + tokenName(tok) + "\n");
+                fileline()->v3error("Expecting include filename. Found: "s + tokenName(tok));
                 goto next_tok;
             }
         }
@@ -1427,12 +1451,12 @@ int V3PreProcImp::getStateToken() {
             if (tok == VP_STRING) {
                 if (!m_off) {
                     m_lastSym.assign(yyourtext(), yyourleng());
-                    error(m_lastSym);
+                    fileline()->v3error(m_lastSym);
                 }
                 statePop();
                 goto next_tok;
             } else {
-                error("Expecting `error string. Found: "s + tokenName(tok) + "\n");
+                fileline()->v3error("Expecting `error string. Found: "s + tokenName(tok));
                 statePop();
                 goto next_tok;
             }
@@ -1479,7 +1503,7 @@ int V3PreProcImp::getStateToken() {
                 statePop();
                 goto next_tok;
             } else if (tok == VP_EOF) {
-                error("`\" not terminated at EOF\n");
+                fileline()->v3error("`\" not terminated at EOF");
                 break;
             } else if (tok == VP_BACKQUOTE) {
                 m_strify += "\\\"";
@@ -1513,7 +1537,7 @@ int V3PreProcImp::getStateToken() {
         case VP_ELSIF: statePush(ps_DEFNAME_ELSIF); goto next_tok;
         case VP_ELSE:
             if (m_ifdefStack.empty()) {
-                error("`else with no matching `if\n");
+                fileline()->v3error("`else with no matching `if");
             } else {
                 const VPreIfEntry lastIf = m_ifdefStack.top();
                 m_ifdefStack.pop();
@@ -1527,7 +1551,7 @@ int V3PreProcImp::getStateToken() {
         case VP_ENDIF:
             UINFO(4, "Endif " << endl);
             if (m_ifdefStack.empty()) {
-                error("`endif with no matching `if\n");
+                fileline()->v3error("`endif with no matching `if");
             } else {
                 const VPreIfEntry lastIf = m_ifdefStack.top();
                 m_ifdefStack.pop();
@@ -1547,7 +1571,7 @@ int V3PreProcImp::getStateToken() {
                 unputString("``");
             }
             if (m_defDepth++ > V3PreProc::DEFINE_RECURSION_LEVEL_MAX) {
-                error("Recursive `define substitution: `" + name);
+                fileline()->v3error("Recursive `define substitution: `" + name);
                 goto next_tok;
             }
             // Substitute
@@ -1618,7 +1642,7 @@ int V3PreProcImp::getStateToken() {
             goto next_tok;
         }
         case VP_EOF:
-            if (!m_ifdefStack.empty()) error("`ifdef not terminated at EOF\n");
+            if (!m_ifdefStack.empty()) fileline()->v3error("`ifdef not terminated at EOF");
             return tok;
         case VP_UNDEFINEALL:
             if (!m_off) {
