@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -755,6 +755,7 @@ class TraceVisitor final : public VNVisitor {
     void createTraceFunctions() {
         // Detect and remove duplicate values
         detectDuplicates();
+        m_graph.removeRedundantEdgesMax(&V3GraphEdge::followAlwaysTrue);
 
         // Simplify & optimize the graph
         if (dumpGraphLevel() >= 6) m_graph.dumpDotFilePrefixed("trace_pre");
@@ -883,10 +884,8 @@ class TraceVisitor final : public VNVisitor {
             }
         }
         VL_RESTORER(m_cfuncp);
-        {
-            m_cfuncp = nodep;
-            iterateChildren(nodep);
-        }
+        m_cfuncp = nodep;
+        iterateChildren(nodep);
     }
     void visit(AstTraceDecl* nodep) override {
         UINFO(8, "   TRACE " << nodep << endl);
@@ -895,9 +894,9 @@ class TraceVisitor final : public VNVisitor {
             nodep->user1p(vertexp);
 
             UASSERT_OBJ(m_cfuncp, nodep, "Trace not under func");
+            VL_RESTORER(m_tracep);
             m_tracep = nodep;
             iterateChildren(nodep);
-            m_tracep = nullptr;
         }
     }
     void visit(AstVarRef* nodep) override {
