@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -173,7 +173,7 @@ class TraceDeclVisitor final : public VNVisitor {
         } else if (!nodep->isTrace()) {
             return "Verilator instance trace_off";
         } else {
-            const string prettyName = varp->prettyName();
+            const string prettyName = nodep->prettyName();
             if (!v3Global.opt.traceUnderscore()) {
                 if (!prettyName.empty() && prettyName[0] == '_') return "Leading underscore";
                 if (prettyName.find("._") != string::npos) return "Inlined leading underscore";
@@ -434,10 +434,14 @@ class TraceDeclVisitor final : public VNVisitor {
 
         // Save the hierarchical names of interface references that reference this scope
         const AstCell* const cellp = nodep->aboveCellp();
-        if (cellp && VN_IS(cellp->modp(), Iface)) {
+        if (cellp
+            && VN_IS(cellp->modp(), Iface)
+            // Exclude classes that were inside interfaces, the cell's modp is an interface,
+            // but the nodep->module is the Class
+            && VN_IS(nodep->modp(), Iface)) {
             const size_t lastDot = path.find_last_of('.');
             UASSERT_OBJ(lastDot != string::npos, nodep,
-                        "Expected an interface scope name to have at least one dot");
+                        "Expected an interface scope name to have at least one dot: " << path);
             const std::string parentPath = path.substr(0, lastDot + 1);
 
             for (AstIntfRef *intfRefp = cellp->intfRefsp(), *nextp; intfRefp; intfRefp = nextp) {
